Package: apsimx
Title: Inspect, Read, Edit and Run 'APSIM' "Next Generation" and
        'APSIM' Classic
Version: 2.8.235
Authors@R: person("Fernando", "Miguez", email = "femiguez@iastate.edu", role = c("aut", "cre"), comment = c(ORCID = "0000-0002-4627-8329"))
Description: The functions in this package inspect, read, edit and run files for 'APSIM' "Next Generation" ('JSON')
             and 'APSIM' "Classic" ('XML'). The files with an 'apsim' extension correspond to
	     'APSIM' Classic (7.x) - Windows only - and the ones with an 'apsimx' extension correspond to 'APSIM' "Next Generation".
	     For more information about 'APSIM' see (<https://www.apsim.info/>) and for 'APSIM'
	     next generation (<https://apsimnextgeneration.netlify.app/>). 
Depends: R (>= 4.0.0)
License: GPL-3
Encoding: UTF-8
VignetteBuilder: knitr
BugReports: https://github.com/femiguez/apsimx/issues
Imports: DBI, jsonlite, knitr, RSQLite, tools, utils, xml2
Suggests: BayesianTools, chirps, datasets, daymetr, future, ggplot2,
        GSODR, listviewer, maps, metrica, mvtnorm, nasapower, nloptr,
        parallelly, reactR, rmarkdown, sensitivity, soilDB, sp, spData,
        sf, ucminf
LazyData: true
RoxygenNote: 7.3.2
NeedsCompilation: no
Packaged: 2025-03-09 22:04:37 UTC; fernandomiguez
Author: Fernando Miguez [aut, cre] (<https://orcid.org/0000-0002-4627-8329>)
Maintainer: Fernando Miguez <femiguez@iastate.edu>
Repository: CRAN
Date/Publication: 2025-03-10 05:40:02 UTC
Built: R 4.3.3; ; 2025-03-10 06:13:24 UTC; unix
