Package: apsimx
Title: Inspect, Read, Edit and Run 'APSIM' "Next Generation" and
        'APSIM' Classic
Version: 2.8.271
Authors@R: person("Fernando", "Miguez", email = "femiguez@iastate.edu", role = c("aut", "cre"), comment = c(ORCID = "0000-0002-4627-8329"))
Description: The functions in this package inspect, read, edit and run files for 'APSIM' "Next Generation" ('JSON')
             and 'APSIM' "Classic" ('XML'). The files with an 'apsim' extension correspond to
	     'APSIM' Classic (7.x) - Windows only - and the ones with an 'apsimx' extension correspond to 'APSIM' "Next Generation".
	     For more information about 'APSIM' see (<https://www.apsim.info/>) and for 'APSIM'
	     next generation (<https://apsimnextgeneration.netlify.app/>). 
Depends: R (>= 4.0.0)
License: GPL-3
Encoding: UTF-8
VignetteBuilder: knitr
BugReports: https://github.com/femiguez/apsimx/issues
Imports: DBI, jsonlite, knitr, RSQLite, tools, utils, xml2
Suggests: BayesianTools, chirps, datasets, daymetr, future, ggplot2,
        GSODR, listviewer, maps, metrica, mgcv, mvtnorm, nasapower,
        nloptr, parallelly, reactR, rmarkdown, sensitivity, soilDB, sp,
        spData, sf, ucminf
LazyData: true
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-07-08 17:02:15 UTC; femiguez
Author: Fernando Miguez [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-4627-8329>)
Maintainer: Fernando Miguez <femiguez@iastate.edu>
Repository: CRAN
Date/Publication: 2026-07-13 23:40:56 UTC
