# Generated by roxygen2: do not edit by hand

S3method("$<-",met)
S3method(coef,optim_apsim)
S3method(confint,optim_apsim)
S3method(plot,met)
S3method(plot,met_mrg)
S3method(plot,out_mrg)
S3method(plot,soil_profile)
S3method(plot,soil_profile_mrg)
S3method(print,met)
S3method(print,met_mrg)
S3method(print,optim_apsim)
S3method(print,out_mrg)
S3method(print,sens_apsim)
S3method(print,soil_profile_mrg)
S3method(summary,met)
S3method(summary,sens_apsim)
S3method(vcov,optim_apsim)
export(add_column_apsim_met)
export(amp_apsim_met)
export(apsim)
export(apsim.options)
export(apsim_example)
export(apsim_options)
export(apsim_version)
export(apsimx)
export(apsimx.options)
export(apsimx_example)
export(apsimx_filetype)
export(apsimx_options)
export(apsimx_soil_profile)
export(as_apsim_met)
export(auto_detect_apsim_examples)
export(auto_detect_apsimx_examples)
export(available_water_content)
export(carbon_stocks)
export(check_apsim_met)
export(check_apsimx)
export(check_apsimx_soil_profile)
export(compare_apsim)
export(compare_apsim_met)
export(compare_apsim_soil_profile)
export(date2doy)
export(doy2date)
export(edit_apsim)
export(edit_apsim_replace_soil_profile)
export(edit_apsim_xml)
export(edit_apsimx)
export(edit_apsimx_batch)
export(edit_apsimx_replace_soil_profile)
export(edit_apsimx_replacement)
export(extract_data_apsimx)
export(extract_values_apsimx)
export(get_apsimx_json)
export(get_chirps_apsim_met)
export(get_daymet2_apsim_met)
export(get_daymet_apsim_met)
export(get_gsod_apsim_met)
export(get_iem_apsim_met)
export(get_iemre_apsim_met)
export(get_isric_soil_profile)
export(get_power_apsim_met)
export(get_slga_soil)
export(get_slga_soil_profile)
export(get_ssurgo_soil_profile)
export(get_ssurgo_tables)
export(get_worldmodeler_apsim_met)
export(get_worldmodeler_soil_profile)
export(grep_json_list)
export(impute_apsim_met)
export(initialwater_parms)
export(insert_replacement_node)
export(inspect_apsim)
export(inspect_apsim_xml)
export(inspect_apsimx)
export(inspect_apsimx_json)
export(inspect_apsimx_replacement)
export(mcmc.apsim.env)
export(mcmc.apsimx.env)
export(napad_apsim_met)
export(optim_apsim)
export(optim_apsimx)
export(read_apsim)
export(read_apsim_all)
export(read_apsim_met)
export(read_apsim_soils)
export(read_apsimx)
export(read_apsimx_all)
export(remove_column_apsim_met)
export(sens_apsim)
export(sens_apsimx)
export(soilorganicmatter_parms)
export(soilwat_parms)
export(solutes_parms)
export(ssurgo2sp)
export(swim_parms)
export(tav_apsim_met)
export(tt_apsim_met)
export(unit_conv)
export(view_apsim)
export(view_apsim_xml)
export(view_apsimx)
export(write_apsim_met)
export(xargs_apsimx)
import(DBI)
import(RSQLite)
import(jsonlite)
import(knitr)
import(xml2)
importFrom(stats,aggregate)
importFrom(stats,anova)
importFrom(stats,coef)
importFrom(stats,cor)
importFrom(stats,cov2cor)
importFrom(stats,deviance)
importFrom(stats,lm)
importFrom(stats,optim)
importFrom(stats,pnorm)
importFrom(stats,qt)
importFrom(stats,quantile)
importFrom(stats,reformulate)
importFrom(stats,sd)
importFrom(stats,setNames)
importFrom(stats,sigma)
importFrom(stats,terms)
importFrom(stats,var)
importFrom(tools,file_ext)
importFrom(tools,file_path_sans_ext)
importFrom(utils,head)
importFrom(utils,packageVersion)
importFrom(utils,read.table)
importFrom(utils,write.table)
